import math
import pygame

from constants import GRAVITY, TILE_SIZE, TARGET_FPS
from helper_function import listfolders


class Boss(pygame.sprite.Sprite):
    def __init__(self, x, y, scale, speed):
        super().__init__()

        # animation
        self.action = 0
        self.frame_index = 0
        self.flip = False
        self.direction = 1
        self.frame_time = pygame.time.get_ticks()
        self.attack_animation_cooldown = 40
        self.animation_cooldown = 100
        self.animation_cycle = []
        self.animation_types = ['walk', 'attack', 'death', 'idle', 'spin_attack']
        self.attack_offset_x_list = [0,0,3,1,1,29,-6,-2,1,-5,-12,-13,0,0,33,3,3,2,1,-19,1,2,2,-11,7,3,-19,-5,0,0]
        self.spin_attack_offset_x_list = [0,0,0,0,12,-2,0,1,1,-16,-3,4,1,-12,15,1,-13,3,5,1,-13,16,-2,-1,-13,1,14,0,0,0]
        #self.spin_attack_offset_x_list = [0,0,0,12,-2,0,1,1,-16,-3,4,1,-12,15,1,-13,3,5,1,-13,16,-2,-1,-13,1,8,4,2,0,0]
        #self.spin_attack_offset_x_list = [14,0,0,0,12,-2,0,1,1,-16,-3,4,1,-12,15,1,-13,3,5,1,-13,16,-2,-1,-13,1,0,0,0,0]

        for animation in self.animation_types:
            temp = []
            num_of_frames = len(listfolders(f'images/enemy/Boss/{animation}'))
            hit_animation = animation + "_hit"
            for i in range(num_of_frames):
                file_name = "tile0"
                if i < 10:
                    file_name += str(0)
                file_name += str(i)
                img = pygame.image.load(f'images/enemy/Boss/{animation}/{file_name}.png').convert_alpha()
                hit_img = pygame.image.load(f'images/enemy/Boss/{hit_animation}/{file_name}.png').convert_alpha()
                right_img = pygame.transform.scale(img, (img.get_width() * scale, img.get_height() * scale))
                left_img = pygame.transform.flip(right_img, True, False)
                hit_right_img = pygame.transform.scale(hit_img, (hit_img.get_width() * scale, img.get_height() * scale))
                hit_left_img = pygame.transform.flip(hit_right_img, True, False)
                offset_x = 0
                if animation == 'attack':
                    offset_x = self.attack_offset_x_list[i]
                elif animation == 'spin_attack':
                    offset_x = self.spin_attack_offset_x_list[i]
                temp.append((right_img, left_img, hit_right_img, hit_left_img, offset_x))
            self.animation_cycle.append(temp)
        self.image = self.animation_cycle[self.action][self.frame_index][0]
        self.image_offset_x = self.animation_cycle[self.action][self.frame_index][4]

        # movement
        self.speed = speed
        self.jump = False
        self.in_air = True
        self.velocity_y = 0

        # rectangle
        self.rect = self.image.get_rect()
        self.rect.center = (x, y)
        self.width = self.image.get_width()
        self.height = self.image.get_height()

        # attack
        self.alive = True
        self.attack = False
        self.regular_attack = True
        self.spin_attack = False
        self.is_hit = False
        #self.done_attacking = True
        self.hit_cooldown = 0
        self.attack_cooldown = 0  # will be used later
        self.heart = 10
        self.idling = False
        self.idling_counter = 0
        self.moving_counter = 0
        self.vision = pygame.Rect(0, 0, 150, 20)
        self.kill_sprite = False

    def update_animation(self):
        self.image = self.animation_cycle[self.action][self.frame_index][0]
        if self.flip:
            self.image = self.animation_cycle[self.action][self.frame_index][1]
        self.image_offset_x = self.animation_cycle[self.action][self.frame_index][4]
        #self.image = self.image.copy()

        animation_cd = self.animation_cooldown
        if self.action == 4 or self.action == 1:
            animation_cd = self.attack_animation_cooldown

        if pygame.time.get_ticks() - self.frame_time > animation_cd:
            #if self.action == 1:

            if self.frame_index < len(self.animation_cycle[self.action]) - 1:
                if self.action == 4:
                    if self.flip:
                        #pass
                        # when attacking left
                        #offset_sum = sum(self.spin_attack_offset_x_list[:self.frame_index])
                        #print(offset_sum)
                        self.rect.x -= self.image_offset_x
                        #img_rect = self.image.get_rect()
                        #img_rect.x -= self.image_offset_x
                    else:
                        # when attacking right, expand the rect to match the image width
                        #offset_sum = sum(self.spin_attack_offset_x_list[:self.frame_index])
                        self.rect.width += self.image_offset_x
                    #self.done_attacking = False
                    #Keep attack flag on:
                    self.attack = True
                elif self.action == 1:
                    if self.flip:
                        #pass
                        # when attacking left
                        self.rect.x -= self.image_offset_x
                        #img_rect = self.image.get_rect()
                        #img_rect.x -= self.image_offset_x
                    else:
                        # when attacking right, expand the rect to match the image width
                        #offset_sum = sum(self.attack_offset_x_list[:self.frame_index])
                        self.rect.width += self.image_offset_x
                    #self.done_attacking = False
                    #Keep attack flag on:
                    self.attack = True
                self.frame_time = pygame.time.get_ticks()
                if self.hit_cooldown > 0 or not self.alive:
                    if self.frame_index % 3 == 0:
                        # change image of every other frame of current action to a flash
                        self.image = self.animation_cycle[self.action][self.frame_index][2]
                        if self.flip:
                            self.image = self.animation_cycle[self.action][self.frame_index][3]
                self.frame_index += 1
            #last frame of animation
            else:
                # the action is the death action, set the frame to be the last frame in the death animation
                if self.action == 2:
                    self.frame_index = len(self.animation_cycle[self.action]) - 1
                    self.kill_sprite = True
                # the action is any other action, go back to the first frame in the animation
                else:
                    #print("done attacking is true")
                    #print(self.frame_index)
                    self.frame_index = 0
                    # the action is the attack action, indicate that the boss is done attacking
                    if self.action == 4 or self.action == 1:
                        print("end of update_animation: ")
                        #self.done_attacking = True
                        self.attack = False
                        self.idling = True
                        self.idling_counter = 100
                        #Switch attack mode
                        self.spin_attack = not self.spin_attack
                        self.regular_attack = not self.regular_attack


    def ai(self, player, world, screen_scroll, dt):
        #print("beginning of ai: ")
        #print(self.done_attacking)
        if self.alive:
            # if self.action == 4 or self.action == 1:
            #     self.attack = True
            #     # Switch attack
            #     self.spin_attack = not self.spin_attack
            #     self.regular_attack = not self.regular_attack
            # else:
            #     self.attack = False
            # Enemy is facing player and sees player
            if self.vision.colliderect(player.rect) and player.alive:
                # Enemy is near player, Attack
                if math.dist(self.rect.center, player.rect.center) < 50: #(or self.attack and self.frame_index > 0) enemy finish attack even if player walks away...
                    #print("Player in vision")
                    #print(sum(self.spin_attack_offset_x_list))
                    self.attack = True
                    # Regular attack or Spin attack
                    # If spin_attack previously, then do a regular attack next
                    if self.spin_attack:
                        #self.spin_attack = False
                        #self.regular_attack = True
                        self.update_action(4)
                        print("Spin attack")
                    # if regular attack previously, then do a spin attack next
                    else:
                        #self.spin_attack = True
                        #self.regular_attack = False
                        self.update_action(1)
                        print("regular attack")
                        #print(sum(self.spin_attack_offset_x_list))
                # Enemy is not close enough to attack, move towardss player if not attacking
                elif not self.attack:
                    if not self.flip:
                         enemy_moving_right = True
                    else:
                         enemy_moving_right = False
                    enemy_moving_left = not enemy_moving_right
                    self.move(enemy_moving_left, enemy_moving_right,world, dt)
                    self.update_action(0)
            # Enemy is not facing player
            else:
                # Enemy is currently attacking
                if self.attack:
                    if self.spin_attack:
                        self.update_action(4)
                    # if regular attack previously, then do a spin attack next
                    else:
                        self.update_action(1)
                    #Do nothing
                    pass
                # Enemy is not attacking
                elif not self.attack:
                    # if idling, count down idling counter and start moving counter
                    if self.idling:
                        self.idling_counter -= 1 * int(round(dt * TARGET_FPS))
                        self.update_action(3)
                        print("idling_counter: ")
                        print(self.idling_counter)
                        #if idling couinter is <=0, then start moving in opposite direction
                        if self.idling_counter <= 0:
                            self.idling = False
                            self.moving_counter = 100

                            self.flip = not self.flip
                            if not self.flip:
                                enemy_moving_right = True
                            else:
                                enemy_moving_right = False
                            enemy_moving_left = not enemy_moving_right
                            self.move(enemy_moving_left, enemy_moving_right, world, dt)
                            self.update_action(0)
                    # else if moving, then count down moving counter and start idling counter
                    else:
                        self.moving_counter -= 1 * int(round(dt * TARGET_FPS))
                        enemy_moving_right = True
                        enemy_moving_left = False
                        if self.flip:
                            enemy_moving_left = True
                            enemy_moving_right = False
                        self.move(enemy_moving_left, enemy_moving_right, world, dt)
                        self.update_action(0)
                        # if moving couinter is <=0, then start idling
                        if self.moving_counter <= 0:
                            self.idling = True
                            self.idling_counter = 100
                            self.update_action(3)

                self.vision.center = (self.rect.centerx + 75 * self.direction, self.rect.centery)
                # pygame.draw.rect(screen, (255, 0, 0), self.vision)
        self.rect.x += screen_scroll

    def move(self, moving_left, moving_right, world, dt):
        dx = 0
        dy = 0

        if moving_left:
            self.flip = True
            self.direction = -1
            dx = -self.speed# * 0.6
        if moving_right:
            self.flip = False
            self.direction = 1
            dx = self.speed

        self.velocity_y += GRAVITY * int(round(dt * TARGET_FPS))
        #dy = int(dy)
        #if self.velocity_y > 10:
        #    self.velocity_y

        dx = dx * int(round(dt * TARGET_FPS))
        dy += self.velocity_y * int(round(dt * TARGET_FPS))

        for tile in world.getObstacleList(False):
            # if tile[1].colliderect(rect) if no care of dx
            if tile[1].colliderect(self.rect.x + dx, self.rect.y, self.width, self.height):
                if self.direction == 1:
                    dx = tile[1].left - self.rect.right
                elif self.direction == -1:
                    dx = tile[1].right - self.rect.left
                self.direction *= -1
                self.flip = not self.flip
            elif tile[1].colliderect(self.rect.x, self.rect.y + dy, self.width, self.height):
                if self.velocity_y >= 0:
                    self.velocity_y = 0
                    self.in_air = False
                    dy = tile[1].top - self.rect.bottom

        self.rect.x += dx
        self.rect.y += dy

        #if dx > 0:
            #print("Moving Right")

    def update(self, player, world, screen_scroll, dt):
        self.ai(player, world, screen_scroll, dt)
        self.check_alive()
        if self.alive:
            if self.rect.colliderect(player):
                print("colliding")

                # 8 possible different positions of enemy and player
                # for each different position: 4 different states
                # 1) Enemy attacking and Player attacking
                #       Enemy spin attack
                #       Enemy regular attack
                # 2) Enemy attacking and Player not attacking
                #       Enemy spin attack
                #       Enemy regular attack
                # 3) Enemy not attacking and player attacking
                # 4) Enemy not attacking and player not attacking


                # <-E P->
                if self.rect.x < player.rect.x and self.direction == -1 and player.direction == 1:
                    if self.attack and player.attack:
                        if self.regular_attack:
                            player.hit()
                        elif self.spin_attack:
                            player.hit()
                    elif self.attack and not player.attack:
                        if self.regular_attack:
                            player.hit()
                        elif self.spin_attack:
                            player.hit()
                    elif not self.attack and player.attack:
                        player.hit()
                    elif not self.attack and not player.attack:
                        player.hit()

                # <-E <-P
                elif self.rect.x < player.rect.x and self.direction == -1 and player.direction == -1:
                    if self.attack and player.attack:
                        if self.regular_attack:
                            self.hit()
                        elif self.spin_attack:
                            self.hit()
                            player.hit()
                    elif self.attack and not player.attack:
                        if self.regular_attack:
                            player.hit()
                        elif self.spin_attack:
                            player.hit()
                    elif not self.attack and player.attack:
                        self.hit()
                    elif not self.attack and not player.attack:
                        player.hit()

                # E-> P->
                elif self.rect.x < player.rect.x and self.direction == 1 and player.direction == 1:
                    if self.attack and player.attack:
                        if self.regular_attack:
                            player.hit()
                        elif self.spin_attack:
                            player.hit()
                    elif self.attack and not player.attack:
                        if self.regular_attack:
                            player.hit()
                        elif self.spin_attack:
                            player.hit()
                    elif not self.attack and player.attack:
                        player.hit()
                    elif not self.attack and not player.attack:
                        player.hit()

                # E-> <-P
                elif self.rect.x < player.rect.x and self.direction == 1 and player.direction == -1:
                    if self.attack and player.attack:
                        if self.regular_attack:
                            self.hit()
                            player.hit()
                        elif self.spin_attack:
                            self.hit()
                            player.hit()
                    elif self.attack and not player.attack:
                        if self.regular_attack:
                            player.hit()
                        elif self.spin_attack:
                            player.hit()
                    elif not self.attack and player.attack:
                        self.hit()
                    elif not self.attack and not player.attack:
                        player.hit()
                # <-P E->
                elif player.rect.x < self.rect.x and player.direction == -1 and self.direction == 1:
                    if self.attack and player.attack:
                        if self.regular_attack:
                            player.hit()
                        elif self.spin_attack:
                            player.hit()
                    elif self.attack and not player.attack:
                        if self.regular_attack:
                            player.hit()
                        elif self.spin_attack:
                            player.hit()
                    elif not self.attack and player.attack:
                        player.hit()
                    elif not self.attack and not player.attack:
                        player.hit()

                # <-P <-E
                elif player.rect.x < self.rect.x and player.direction == -1 and self.direction == -1:
                    if self.attack and player.attack:
                        if self.regular_attack:
                            player.hit()
                        elif self.spin_attack:
                            player.hit()
                    elif self.attack and not player.attack:
                        if self.regular_attack:
                            player.hit()
                        elif self.spin_attack:
                            player.hit()
                    elif not self.attack and player.attack:
                        player.hit()
                    elif not self.attack and not player.attack:
                        player.hit()

                # P-> E->
                elif player.rect.x < self.rect.x and player.direction == 1 and self.direction == 1:
                    if self.attack and player.attack:
                        if self.regular_attack:
                            self.hit()
                        elif self.spin_attack:
                            self.hit()
                            player.hit()
                    elif self.attack and not player.attack:
                        if self.regular_attack:
                            player.hit()
                        elif self.spin_attack:
                            player.hit()
                    elif not self.attack and player.attack:
                        self.hit()
                    elif not self.attack and not player.attack:
                        player.hit()

                # P-> <-E
                elif player.rect.x < self.rect.x and player.direction == 1 and self.direction == -1:
                    if self.attack and player.attack:
                        if self.regular_attack:
                            self.hit()
                            player.hit()
                        elif self.spin_attack:
                            self.hit()
                            player.hit()
                    elif self.attack and not player.attack:
                        if self.regular_attack:
                            player.hit()
                        elif self.spin_attack:
                            player.hit()
                    elif not self.attack and player.attack:
                        self.hit()
                    elif not self.attack and not player.attack:
                        player.hit()
                #P == E
                elif player.rect.x == self.rect.x:
                    if self.attack and player.attack:
                        if self.regular_attack:
                            self.hit()
                            player.hit()
                        elif self.spin_attack:
                            self.hit()
                            player.hit()
                    elif self.attack and not player.attack:
                        if self.regular_attack:
                            player.hit()
                        elif self.spin_attack:
                            player.hit()
                    elif not self.attack and player.attack:
                        self.hit()
                    elif not self.attack and not player.attack:
                        player.hit()

                # if player.alive and player.attack:
                #     # if enemy attacks and is facing in the direction of the player
                #     if self.attack and (self.direction + player.direction == 0):
                #         player.player_hit()
                #     if self.hit_cooldown == 0:
                #         self.heart -= 1
                #         #self.update_action(4)
                #         self.hit_cooldown = 100
                # any other kind of collision
                # elif not player.shield and not player.roll_speed > 0 and not player.attack:
                #    player.player_hit()
            if self.hit_cooldown > 0:
                self.hit_cooldown = self.hit_cooldown - int(round(dt * TARGET_FPS))
        else:
            self.update_action(2)
        self.update_animation()
        if self.kill_sprite:
            self.kill()

    def hit(self):
        if self.hit_cooldown == 0:
            self.heart -= 1
            self.hit_cooldown = 100
            #self.ui_update = True

    def update_action(self, new_action):
        #print("current action")
        #print(self.action)
        if new_action != self.action:
            #if golem is being hit and is in cooldown
            #if new_action != 2 and self.action == 4 and self.hit_cooldown != 0:
            #    self.action = self.action

            # if new action is differetn from the boss's current action is to attack,
            # and the bossi snot done attacking, then keep the current attack action
            # if self.action == 4:
            #     print("current action is 4, is enemy done attacking?")
            #     print(self.done_attacking)

            # Boss's current action is attacking and is not yet down attacking, keep the current animation
            #if (self.action == 4 or self.action == 1) and not self.done_attacking:
                #self.action = self.action
            #else:
            self.action = new_action
            self.frame_index = 0
            self.frame_time = pygame.time.get_ticks()
        else:
            self.action = self.action

    def check_alive(self):
        if self.heart <= 0:
            self.heart = 0
            self.speed = 0
            self.hit_cooldown = 0
            self.alive = False

    def draw(self, displayedSurface):
        displayedSurface.blit(self.image, self.rect)
        # pygame.draw.rect(screen, (0, 0, 0), self.rect, 1)