import pygame

from constants import TILE_SIZE
from helper_function import listfolders
from components.music import campfire_sound


class Item(pygame.sprite.Sprite):
    def __init__(self, x, y):
        pygame.sprite.Sprite.__init__(self)

        # animation
        self.action = 0
        self.frame_index = 0
        self.frame_time = pygame.time.get_ticks()
        self.animation_cooldown = 100
        self.animation_cycle = []
        self.animation_types = ['not_used']
        for animation in self.animation_types:
            temp = []
            num_of_frames = len(listfolders(f'images/Item/FirePlace/{animation}'))
            for i in range(num_of_frames):
                img = pygame.image.load(f'images/Item/FirePlace/{animation}/tile00{i}.png').convert_alpha()
                temp.append(img)
            self.animation_cycle.append(temp)
        self.image = self.animation_cycle[self.action][self.frame_index]

        # rectangle
        self.rect = self.image.get_rect()
        self.rect.midtop = (x + TILE_SIZE // 2, y + (TILE_SIZE - self.image.get_height()))

        # collision
        self.fired = False

    def update(self, player,screen_scroll):
        self.update_animation()
        self.rect.x += screen_scroll
        if pygame.sprite.collide_rect(self, player):
            campfire_sound.play()
        else:
            campfire_sound.stop()
        if not self.fired:
            if pygame.sprite.collide_rect(self, player):
                campfire_sound.play()
                if player.heart < player.max_heart:
                    player.heart += 1
                else:
                    player.heart = player.max_heart
                self.fired = True
        else:
            player.heart

    def update_animation(self):
        self.image = self.animation_cycle[self.action][self.frame_index]
        if pygame.time.get_ticks() - self.frame_time > self.animation_cooldown:
            if self.frame_index < len(self.animation_cycle[self.action]) - 1:
                self.frame_time = pygame.time.get_ticks()
                self.frame_index += 1
            else:
                if self.action == 2:
                    self.frame_index = len(self.animation_cycle[self.action]) - 1
                else:
                    self.frame_index = 0
