import pygame

from constants import *
from helper_function import listfolders, create_text_surf


class King(pygame.sprite.Sprite):
    def __init__(self, x, y, scale, level):
        pygame.sprite.Sprite.__init__(self)
        self.level = level
        self.frame_time = 0
        self.frame_index = 0
        self.boss_idle_cycle = []
        self.king_idle_cycle = []
        boss_num_of_frames = len(listfolders(f'images/enemy/Boss/idle'))
        for i in range(boss_num_of_frames):
            file_name = "tile0"
            if i < 10:
                file_name += str(0)
            file_name += str(i)
            img = pygame.image.load(f'images/enemy/Boss/idle/{file_name}.png').convert_alpha()
            img = pygame.transform.scale(img, (img.get_width() * scale, img.get_height() * scale))
            img = pygame.transform.flip(img, True, False)
            self.boss_idle_cycle.append(img)

        king_num_of_frames = len(listfolders(f'images/npc/king/idle'))
        for j in range(king_num_of_frames):
            img = pygame.image.load(f'images/npc/king/idle/tile{j}.png').convert_alpha()
            img = pygame.transform.scale(img, (img.get_width() * scale, img.get_height() * scale))
            img = pygame.transform.flip(img, True, False)
            self.king_idle_cycle.append(img)

        self.image = self.king_idle_cycle[self.frame_index]
        self.rect = self.image.get_rect()
        self.rect.midtop = (x + TILE_SIZE // 2, y + (TILE_SIZE - self.image.get_height()))

        self.animation_cooldown = 100
        self.meet_player = False
        self.count = 200
        self.next_level = False
        self.dialog_iterator = 0
        self.dialogs = []
        self.dialogs.append((create_text_surf("Well done, my loyal knight...", 'slkscr.ttf', 10, WHITE), (0.5 * self.rect.x, self.rect.y-10)))
        if self.level == 2:
            self.dialogs.append((create_text_surf("You have served me well and defeated all of my minions.", 'slkscr.ttf', 10, RED), (0.35*self.rect.x, self.rect.y-10)))
            self.dialogs.append((create_text_surf("Fool, did you not know that it was me all along?", 'slkscr.ttf', 10, RED), (0.4*self.rect.x, self.rect.y-10)))
            self.dialogs.append((create_text_surf("Prepare to suffer my wrath!", 'slkscr.ttf', 10, RED), (0.5*self.rect.x, self.rect.y-10)))
        self.displayed_dialog = self.dialogs[0][0]
        self.displayed_dialog_pos = self.dialogs[0][1]
        self.dialogs_finished = False

        self.isBoss = False
        self.draw_pos = self.rect


    def update_animation(self):
        if self.dialog_iterator == 0:
            self.image = self.king_idle_cycle[self.frame_index]
            if pygame.time.get_ticks() - self.frame_time > self.animation_cooldown:
                if self.frame_index < len(self.king_idle_cycle) - 1:
                    self.frame_time = pygame.time.get_ticks()
                    self.frame_index += 1
                else:
                    self.frame_index = 0
        else:
            #old_x = self.image.get_rect().x
            if self.frame_index > 15:
                self.frame_index = 15
            self.isBoss = True
            self.image = self.boss_idle_cycle[self.frame_index]
            if pygame.time.get_ticks() - self.frame_time > self.animation_cooldown:
                if self.frame_index < len(self.boss_idle_cycle) - 1:
                    self.frame_time = pygame.time.get_ticks()
                    self.frame_index += 1
                else:
                    self.frame_index = 0


    # if the level is the last level, then
    def update(self, player, screen_scroll, dt):
        #print("King update")
        self.update_animation()
        if player.rect.colliderect(self.rect.x + TILE_SIZE, self.rect.y, self.rect.width, self.rect.height):
            #print("meet king")
            self.meet_player = True
        if self.meet_player:
            print("meet player loop")
            self.count = self.count - int(round(dt * TARGET_FPS))
            if self.count <= 0:
                print("count is 0")
                self.count = 150
                self.dialog_iterator += 1
                if self.dialog_iterator == len(self.dialogs):
                    print("meet player is false and dialogs  are finished")
                    self.meet_player = False
                    self.dialogs_finished = True
                else:
                    self.displayed_dialog = self.dialogs[self.dialog_iterator][0]
                    self.displayed_dialog_pos = self.dialogs[self.dialog_iterator][1]
                    print(self.dialog_iterator)
                    print(len(self.dialogs))

        if self.dialogs_finished:
            print("dialogs finished, next level is true")
            self.next_level = True
        self.rect.x += screen_scroll


    def draw(self, surface):
        if self.isBoss:
            self.draw_pos = self.rect.move(40,12)
        surface.blit(pygame.transform.flip(self.image, False, False), self.draw_pos)

    def draw_dialog(self, surface):
        #print("Draw Dialog")
        surface.blit(self.displayed_dialog, self.displayed_dialog_pos)

