import pygame

from constants import *
from helper_function import listfolders
from components.music import player_attack_sound


class Player(pygame.sprite.Sprite):
    def __init__(self, x, y, scale, speed, joystick=None):
        super().__init__()

        # animation
        #self.player_attack_sound = pygame.mixer.Sound('audio/action/player_attack.wav')
        #self.player_attack_sound.set_volume(0.1)
        self.joystick = joystick
        self.action = 0
        self.frame_index = 0
        self.flip = False
        self.direction = 1
        self.frame_time = pygame.time.get_ticks()
        self.animation_cooldown = 100
        self.animation_cycle = []
        self.animation_types = ['idle', 'walk', 'jump', 'attack', 'death', 'shield', 'roll']
        #self.attack_offset_x_list = [0, 0, 0, 0, 19, 0, 0, -4, -12, 15, -2, 1, -6, -6, -5, 0, 34, -12, -22, 0, 0, 0]
        self.attack_offset_x_list = [0, 0, 0, 0, 19, 0, 0, -4, -12, 15, -2, 1, -6, -6, -5, -10, 0, 34, -12, -12, 0, 0]

        #self.tmp = pygame.Surface(right_img.get_rect().size, pygame.SRCALPHA)
                #tmp.fill((255, 255, 255, 10))

        for animation in self.animation_types:
            temp = []
            num_of_frames = len(listfolders(f'images/player/knight/{animation}'))
            hit_animation = animation + "_hit"
            for i in range(num_of_frames):
                img = pygame.image.load(f'images/player/knight/{animation}/tile{i}.png').convert_alpha()
                hit_img = pygame.image.load(f'images/player/knight/{hit_animation}/tile{i}.png').convert_alpha()
                right_img = pygame.transform.scale(img, (img.get_width() * scale, img.get_height() * scale))
                left_img = pygame.transform.flip(right_img, True, False)
                hit_right_img = pygame.transform.scale(hit_img, (hit_img.get_width() * scale, img.get_height() * scale))
                hit_left_img = pygame.transform.flip(hit_right_img, True, False)
                offset_x = 0
                if animation == 'attack':
                    offset_x = self.attack_offset_x_list[i]
                temp.append((right_img, left_img, hit_right_img, hit_left_img, offset_x))
            self.animation_cycle.append(temp)

        self.image = self.animation_cycle[self.action][self.frame_index][0]
        self.image_offset_x = self.animation_cycle[self.action][self.frame_index][4]

        # rectangle
        self.rect = self.image.get_rect()
        self.rect.center = (x, y)
        self.width = self.image.get_width()
        self.height = self.image.get_height()

        # movement
        self.speed = speed
        self.jump = False
        self.in_air = True
        self.velocity_y = 0
        self.roll = False
        self.roll_speed = 0

        # attack
        self.alive = True
        self.attack = False
        self.attack_cooldown = 1000
        self.attack_time = pygame.time.get_ticks()
        self.attack_animation_cooldown = 400
        self.attack_animation_time = pygame.time.get_ticks()
        self.is_hit = False
        self.hit_cooldown = 0
        self.heart = 5
        self.max_heart = 10

        # shield
        self.shield = False
        self.shieldCount = 2
        self.max_shield = 5

    def increase_shield(self):
        if self.shieldCount < self.max_shield:
            self.shieldCount += 1

    def move(self, moving_left, moving_right, world, spike_group, dt, scroll_total, do_not_scroll):
        dx = 0
        dy = 0
        screen_scroll = 0
        if self.action == 6 and self.roll_speed > 0 and not self.in_air:
            if not self.flip:
                dx = int(round(self.speed + self.roll_speed))
            elif self.flip:
                dx = int(round(-self.speed - self.roll_speed))# * 0.8
        else:
            if moving_left:
                self.flip = True
                self.direction = -1
                dx = -self.speed# * 0.8
            if moving_right:
                self.flip = False
                self.direction = 1
                dx = self.speed

        if self.jump and not self.in_air:
            self.velocity_y = PLAYER_JUMP_VEL
            self.jump = False
            self.in_air = True
        #print("PLAYER VELOCITY Y BEFORE GRAVITY")
        #print(self.velocity_y)
        self.velocity_y += GRAVITY * int(round(dt * TARGET_FPS))
        #print("PLAYER VELOCITY Y AFTER GRAVITY")
        #print(self.velocity_y)

        dx = dx * int(round(dt * TARGET_FPS))
        #print("PLAYER DY BEFORE ADD")
        #print(dy)
        if dt > 0.1:
            dt = 1 / 60
        dy += self.velocity_y * int(round(dt * TARGET_FPS))
        #print("PLAYER DY AFTER ADD")
        #print(dy)

        # check collision with the floor
        # if self.rect.bottom + dy > 300:
        # dy = 300 - self.rect.bottom  # equals to 0
        # self.in_air = False
        for tile in world.getObstacleList(True):
            # if tile[1].colliderect(rect) if no care of dx
            if tile[1].colliderect(self.rect.x + dx, self.rect.y, self.width, self.height):
                dx = 0
                self.roll = False
            elif tile[1].colliderect(self.rect.x, self.rect.y + dy, self.width, self.height):
                #print("TILE COLLIDING WITH BELOW")
                #print(tile[1])
                if self.velocity_y < 0:
                    self.velocity_y = 0
                    dy = tile[1].bottom - self.rect.top
                #Falling on top of tile,
                elif self.velocity_y >= 0:
                    #print(tile[1].top)
                    #print(self.rect.bottom)
                    #print(self.velocity_y)
                    self.velocity_y = 0
                    self.in_air = False
                    dy = tile[1].top - self.rect.bottom

        for spike in spike_group:
            if spike.rect.colliderect(self.rect.x, self.rect.y + dy, self.width, self.height):
                if self.velocity_y >= 0:
                    self.velocity_y = 0
                    self.in_air = False
                    dy = tile[1].top - self.rect.bottom
                self.heart = 0
                #self.ui_update = True
        # if dx > 0:
        # print("action: " + str(self.action))
        # print("frame_index: " + str(self.frame_index))
        # print("dx: " + str(dx))

        self.rect.x += dx
        self.rect.y += dy

        # screen scroll
        #print(self.rect.right)
        #print(SURFACE_WIDTH - SCROLL_THRESH)
        if not do_not_scroll:
            if self.rect.right > SURFACE_WIDTH - SCROLL_THRESH and dx > 0:
                # print("player_right")
                # print(self.rect.right)
                # print("WIDTH - THRESH")
                # print(SURFACE_WIDTH - SCROLL_THRESH)
                # print("player_left")
                # print(self.rect.left)
                # print("THRESH")
                # print(SCROLL_THRESH)
                # print("abs(dx)")
                # print(abs(dx))
                self.rect.x -= dx
                screen_scroll = -dx
                scroll_total += screen_scroll
                #print(screen_scroll)
                #print(scroll_total)

        return screen_scroll, scroll_total

    def update_animation(self):
        self.image = self.animation_cycle[self.action][self.frame_index][0]
        if self.flip:
            self.image = self.animation_cycle[self.action][self.frame_index][1]
        self.image_offset_x = self.animation_cycle[self.action][self.frame_index][4]

        #self.image = self.image.copy()

        if self.action == 3:
            if pygame.time.get_ticks() - self.frame_time > self.animation_cooldown / 200:
                if self.frame_index < len(self.animation_cycle[self.action]) - 1:
                    self.frame_time = pygame.time.get_ticks()
                    # when attacing left, move rect to left
                    if self.flip:
                        self.rect.x -= self.image_offset_x
                        # print(self.rect)
                    else:
                        # print(self.rect)
                        # pass
                        # when attacking right, expand the rect to match the image width
                        self.rect.width += self.image_offset_x
                        #print(self.rect)
                    if self.hit_cooldown > 0:
                        if self.frame_index % 5 == 0:
                            self.image = self.animation_cycle[self.action][self.frame_index][2]
                            if self.flip:
                                self.image = self.animation_cycle[self.action][self.frame_index][3]
                    self.frame_index += 1
                else:
                    self.frame_index = 0
                    self.attack = False
        elif self.action == 6:
            if pygame.time.get_ticks() - self.frame_time > self.animation_cooldown / 3:
                if self.frame_index < len(self.animation_cycle[self.action]) - 1:
                    self.frame_time = pygame.time.get_ticks()
                    if self.hit_cooldown > 0:
                        if self.frame_index % 5 == 0:
                            self.image = self.animation_cycle[self.action][self.frame_index][2]
                            if self.flip:
                                self.image = self.animation_cycle[self.action][self.frame_index][3]
                    self.frame_index += 1
                    self.roll = True
                    self.roll_speed = ROLL_SPEED
                else:
                    #print("end of roll")
                    self.frame_index = 0
                    self.roll = False
                    self.roll_speed = 0
        else:
            if pygame.time.get_ticks() - self.frame_time > self.animation_cooldown:
                if self.frame_index < len(self.animation_cycle[self.action]) - 1:
                    self.frame_time = pygame.time.get_ticks()
                    if self.hit_cooldown > 0:
                        if self.frame_index % 7 != 0:
                            self.image = self.animation_cycle[self.action][self.frame_index][2]
                            if self.flip:
                                self.image = self.animation_cycle[self.action][self.frame_index][3]
                    self.frame_index += 1
                else:
                    if self.action == 4:
                        self.frame_index = len(self.animation_cycle[self.action]) - 1
                    else:
                        self.frame_index = 0

    def update_action(self, new_action):
        if new_action != self.action:
            attack_animation_cd = self.attack_animation_cooldown
            if new_action == 3 or new_action == 6:
                attack_animation_cd = self.attack_animation_cooldown / 200
            if pygame.time.get_ticks() - self.attack_animation_time > attack_animation_cd:
                self.attack_animation_time = pygame.time.get_ticks()
                self.action = new_action
                self.frame_index = 0
                self.frame_time = pygame.time.get_ticks()

    def check_alive(self):
        if self.heart <= 0:
            self.heart = 0
            self.speed = 0
            self.alive = False
            self.update_action(4)
        if self.rect.bottom > SURFACE_HEIGHT:
            print("player is greater than the bottom of surface")
            self.heart = 0
            #self.ui_update = True

    def update(self, dt):
        self.update_animation()
        if self.hit_cooldown > 0:
            self.hit_cooldown -= int(round((dt * TARGET_FPS)))
        self.check_alive()

    def hit(self):
        if self.hit_cooldown == 0 and not self.shield and not self.roll:
            self.heart -= 1
            self.hit_cooldown = 100
            player_attack_sound.play()
            #self.ui_update = True

    def attacking(self):
        if pygame.time.get_ticks() - self.attack_time > self.attack_cooldown:
            self.attack_time = pygame.time.get_ticks()
            self.attack = True
            self.update_action(3)
        elif pygame.time.get_ticks() - self.attack_animation_time > self.attack_animation_cooldown:
            self.attack_animation_time = pygame.time.get_ticks()
            self.attack = False

    def shielding(self, keyDown):
        if keyDown and self.shieldCount > 0:
            self.shieldCount -= 1
            self.shield = True
            self.update_action(5)
            #self.ui_update = True
        elif not keyDown:
            self.shield = False
            self.hit_cooldown = 100

    def draw(self, displayedSurface):
        displayedSurface.blit(self.image, self.rect)
        #    screen.blit(pygame)
        # screen.blit(self.image, self.rect)
        # pygame.draw.rect(screen, (0, 0, 0), self.rect, 1)
