import csv
import pygame

from constants import ROWS, COLS, TILE_TYPES, TILE_SIZE, PLAYER_SPEED, WOLF_SPEED, GOLEM_SPEED, EDGE_TILE, BOSS_SPEED, SURFACE_HEIGHT, SURFACE_WIDTH
from components.decoration import Decoration
from components.item import Item
from components.king import King
from components.player import Player
from helper_function import draw_bg
from components.spike import Spike
from components.wolf import Wolf
from components.golem import Golem
from components.boss import Boss


CHUNK_SIZE = 5

class World():
    def __init__(self):
        #self.joystick = joystick
        self.game_map = {}
        self.world_data = []
        self.chunks_to_draw = {}
        self.level_length = COLS
        self.current_chunk_range = []
        self.player_obstacle_list = []
        self.enemy_obstacle_list = []
        self.all_sprites = pygame.sprite.Group()
        self.player_group = pygame.sprite.Group()
        self.enemy_group = pygame.sprite.Group()
        self.item_boxes_group = pygame.sprite.Group()
        self.spike_group = pygame.sprite.Group()
        self.decoration_group = pygame.sprite.Group()
        self.king_group = pygame.sprite.Group()

        # map
        self.img_list = []
        for x in range(TILE_TYPES):
            img = pygame.image.load(f'map/tile/{x}.png').convert_alpha()
            img = pygame.transform.scale(img, (TILE_SIZE, TILE_SIZE))
            self.img_list.append(img)
        self.sky_img = pygame.image.load('images/Background/spr_Sky_strip.png').convert_alpha()
        #self.img2 = pygame.image.load('images/Background/map.png').convert_alpha()
        self.sky_img = pygame.transform.scale(self.sky_img, (self.sky_img.get_width() * 0.67, self.sky_img.get_height() * 0.67))

        self.obstacle_surface = pygame.Surface([SURFACE_WIDTH,SURFACE_WIDTH], pygame.SRCALPHA, 32).convert_alpha()
        self.obstacle_surface_loaded = False

    def reset_level(self):
        self.player_group.empty()
        self.enemy_group.empty()
        self.item_boxes_group.empty()
        self.spike_group.empty()
        self.decoration_group.empty()
        self.king_group.empty()

        data = []
        for row in range(ROWS):
            r = [-1] * COLS
            data.append(r)

        return data

    def load_data(self, csv_file):
        for row in range(ROWS):
            temp = [-1] * COLS
            self.world_data.append(temp)

        with open(csv_file, newline='') as csvfile:
            reader = csv.reader(csvfile, delimiter=',')
            for x, row in enumerate(reader):
                #print(f'x:{x},row:{row} \n')
                for y, tile in enumerate(row):
                    #print(f'x:{x},y:{y},tile:{tile} \n')
                    self.world_data[x][y] = int(tile)

    def generate_chunk(self, x, y):
        chunk_data = []
        for x_pos in range(CHUNK_SIZE):
            for y_pos in range(CHUNK_SIZE):
                target_x = x * CHUNK_SIZE + x_pos
                target_y = y * CHUNK_SIZE + y_pos
                tile = self.world_data[target_x][target_y]
                chunk_data.append([[target_x, target_y], tile])
        return chunk_data

    def process_data(self, level, joystick=None):
        player = None
        boss = None
        self.current_chunk_range = []
        for y in range(6):
            target_chunk_y = y
            self.current_chunk_range.append(target_chunk_y)
            #print(self.current_chunk_range)
            for x in range(3):
                target_chunk_x = x
                target_chunk = str(target_chunk_x) + ';' + str(target_chunk_y)
                if target_chunk not in self.game_map:
                    self.game_map[target_chunk] = self.generate_chunk(target_chunk_x, target_chunk_y)
                    tiles_to_draw = []
                    for map_coord_tile_data in self.game_map[target_chunk]:
                        coord_x = map_coord_tile_data[0][0]
                        coord_y = map_coord_tile_data[0][1]
                        pos_x = coord_y  # y coordinate on the csv grid determines he x position of the tile
                        pos_y = coord_x  # x coordinate on the csv grid determines the y position of the tile
                        pos_x_scaled = pos_x * TILE_SIZE
                        tile = map_coord_tile_data[1]
                        if tile >= 0:
                            img = self.img_list[tile]
                            img_rect = img.get_rect()
                            img_rect.x = pos_x_scaled
                            img_rect.y = pos_y * TILE_SIZE
                            tile_data = (img, img_rect, tile)
                            if 0 <= tile <= 10:
                                tiles_to_draw.append(tile_data)
                            elif tile == 11:
                                enemy = Wolf(pos_x_scaled, pos_y * TILE_SIZE, 1, WOLF_SPEED)
                                self.all_sprites.add(enemy)
                                self.enemy_group.add(enemy)
                            elif tile == 12:
                                tiles_to_draw.append(tile_data)
                            elif tile == 13:
                                king = King(pos_x_scaled, pos_y * TILE_SIZE, 1, level)
                                self.all_sprites.add(king)
                                self.king_group.add(king)
                            elif 14 <= tile <= 16 or 28 <= tile <= 33:
                                spike = Spike(img, pos_x_scaled, pos_y * TILE_SIZE)
                                self.all_sprites.add(spike)
                                self.spike_group.add(spike)
                            elif tile == 17:
                                player = Player(pos_x_scaled, pos_y * TILE_SIZE, 1, PLAYER_SPEED, joystick)
                                self.all_sprites.add(player)
                                self.player_group.add(player)
                            elif 18 <= tile <= 21:
                                tiles_to_draw.append(tile_data)
                            elif tile == 22:
                                item_box = Item(pos_x_scaled, pos_y * TILE_SIZE)
                                self.all_sprites.add(item_box)
                                self.item_boxes_group.add(item_box)
                            elif tile == 24:
                                decoration = Decoration(img, pos_x_scaled, pos_y * TILE_SIZE)
                                self.all_sprites.add(decoration)
                                self.decoration_group.add(decoration)
                            elif tile == 25:
                                enemy = Golem(pos_x_scaled, pos_y * TILE_SIZE, 1, GOLEM_SPEED)
                                self.all_sprites.add(enemy)
                                self.enemy_group.add(enemy)
                            elif tile == EDGE_TILE:
                                tiles_to_draw.append(tile_data)
                            elif tile == 27:
                                boss = Boss(pos_x_scaled, pos_y * TILE_SIZE, 1, BOSS_SPEED)
                                self.all_sprites.add(boss)
                    self.chunks_to_draw[target_chunk] = tiles_to_draw
        self.updateObstacleLists(True, 0)
        return player, boss

    def update(self, level, scroll_total, screen_scroll):
        self.current_chunk_range = []
        #for x in range(3):
        for y in range(7):
            target_chunk_y = y - 1 + int(round((-scroll_total + CHUNK_SIZE * TILE_SIZE)) / (CHUNK_SIZE * TILE_SIZE))
            if target_chunk_y >= 30:
                break
            self.current_chunk_range.append(target_chunk_y)
            for x in range(3):
                target_chunk_x = x
                target_chunk = str(target_chunk_x) + ';' + str(target_chunk_y)
                if target_chunk not in self.game_map:
                    self.game_map[target_chunk] = self.generate_chunk(target_chunk_x, target_chunk_y)
                    #print(self.game_map[target_chunk])
                    #print(scroll_total)
                    tiles_to_draw = []
                    for map_coord_tile_data in self.game_map[target_chunk]:
                        coord_x = map_coord_tile_data[0][0]
                        coord_y = map_coord_tile_data[0][1]
                        pos_x = coord_y #y coordinate on the csv grid determines he x position of the tile
                        pos_y = coord_x #x coordinate on the csv grid determines the y position of the tile
                        pos_x_scaled = pos_x * TILE_SIZE + scroll_total
                        tile = map_coord_tile_data[1]
                        if tile >= 0:
                            img = self.img_list[tile]
                            img_rect = img.get_rect()
                            img_rect.x = pos_x_scaled
                            img_rect.y = pos_y * TILE_SIZE
                            tile_data = (img, img_rect, tile)
                            if 0 <= tile <= 10:
                                tiles_to_draw.append(tile_data)
                            elif tile == 11:
                                enemy = Wolf(pos_x_scaled, pos_y * TILE_SIZE, 1, WOLF_SPEED)
                                self.all_sprites.add(enemy)
                                self.enemy_group.add(enemy)
                            elif tile == 12:
                                tiles_to_draw.append(tile_data)
                            elif tile == 13:
                                king = King(pos_x_scaled, pos_y * TILE_SIZE, 1, level)
                                print(king)
                                self.all_sprites.add(king)
                                self.king_group.add(king)
                            elif 14 <= tile <= 16 or 28 <= tile <= 33:
                                spike = Spike(img, pos_x_scaled, pos_y * TILE_SIZE)
                                self.all_sprites.add(spike)
                                self.spike_group.add(spike)
                            #elif tile == 17:
                            #    player = Player(pos_x_scaled, pos_y * TILE_SIZE, 1, PLAYER_SPEED, self.joystick)
                            #    self.all_sprites.add(player)
                            #    self.player_group.add(player)
                            elif 18 <= tile <= 21:
                                tiles_to_draw.append(tile_data)
                            elif tile == 22:
                                item_box = Item(pos_x_scaled, pos_y * TILE_SIZE)
                                self.all_sprites.add(item_box)
                                self.item_boxes_group.add(item_box)
                            elif tile == 24:
                                decoration = Decoration(img, pos_x_scaled, pos_y * TILE_SIZE)
                                self.all_sprites.add(decoration)
                                self.decoration_group.add(decoration)
                            elif tile == 25:
                                enemy = Golem(pos_x_scaled, pos_y * TILE_SIZE, 1, GOLEM_SPEED)
                                self.all_sprites.add(enemy)
                                self.enemy_group.add(enemy)
                            elif tile == EDGE_TILE:
                                tiles_to_draw.append(tile_data)
                                #"the edge tile added to the dict:"
                                #print(tile_data)
                            elif tile == 27:
                                boss = Boss(pos_x_scaled, pos_y * TILE_SIZE, 1, BOSS_SPEED)
                                self.all_sprites.add(boss)
                    self.chunks_to_draw[target_chunk] = tiles_to_draw
        self.updateObstacleLists(False, screen_scroll)

    #OPTIMIZE: instead of drawing every tile each time, 
    #draw the tiles to the surface once, then scroll the entire surface
    #including the decorations and spikes
    # If obstacle list is being updated
    def draw(self, displayedSurface):
        obstacles = self.getObstacleList(False)
        #print("NUMBER OF OBSTACLE BLITS")
        #print(len(obstacles))
        for tile_data in obstacles:
            #print(screen_scroll)
            displayedSurface.blit(tile_data[0], tile_data[1])


    def getObstacleList(self, is_player):
        if is_player:
            return self.player_obstacle_list
        else:
            return self.enemy_obstacle_list

    def updateObstacleLists(self, isFirstLoad, screen_scroll):
        if screen_scroll < 0 or isFirstLoad:
            print("UPDATING OBSTACLE LIST")
            self.obstacle_surface_loaded = False
            self.player_obstacle_list = []
            self.enemy_obstacle_list = []
            #print(self.current_chunk_range)
            for chunk_y in self.current_chunk_range:
                for chunk_x in range(3):
                    chunk = str(chunk_x) + ';' + str(chunk_y)
                    for tile_data in self.chunks_to_draw[chunk]:
                        # skip edge tiles for the player so player does not collide with them
                        if tile_data[2] == EDGE_TILE:
                            self.enemy_obstacle_list.append(tile_data)
                        else:
                            self.enemy_obstacle_list.append(tile_data)
                            self.player_obstacle_list.append(tile_data)
            for tile_data in self.getObstacleList(False):
                tile_data[1][0] += screen_scroll
