import pygame

from constants import *
from helper_function import draw_bg, create_text_surf, listfolders

class Knight(pygame.sprite.Sprite):
    def __init__(self, x, y, scale):
        super().__init__()

        # animation
        self.action = 1
        self.frame_index = 0
        self.frame_time = pygame.time.get_ticks()
        self.animation_cooldown = 50
        self.animation_cycle = []
        self.animation_types = ['idle', 'walk', 'attack']
        for animation in self.animation_types:
            temp = []
            num_of_frames = len(listfolders(f'images/player/knight/{animation}'))
            for i in range(num_of_frames):
                img = pygame.image.load(f'images/player/knight/{animation}/tile{i}.png').convert_alpha()
                img = pygame.transform.scale(img, (img.get_width() * scale, img.get_height() * scale))
                temp.append(img)
            self.animation_cycle.append(temp)
        self.image = self.animation_cycle[self.action][self.frame_index]

        # rectangle
        self.original_x = x
        self.original_y = y
        self.rect = self.image.get_rect()
        self.rect.center = (x, y)
        self.done_attacking = False

    def reset(self):
        self.rect = pygame.Rect(self.original_x, self.original_y, self.rect.width, self.rect.height)
        self.action = 1
        self.frame_index = 0

    def move(self, end):
        if self.action == 1:
            self.rect.x = self.rect.x + 4
        if self.rect.x >= end:
            self.action = 0
            # being set to 0 right after attacking key

    def update_animation(self):
        self.image = self.animation_cycle[self.action][self.frame_index]
        if pygame.time.get_ticks() - self.frame_time > self.animation_cooldown:
            if self.frame_index < len(self.animation_cycle[self.action]) - 1:
                self.frame_time = pygame.time.get_ticks()
                self.frame_index += 1
                # print("Knight")
                # print(self.action)
                # print(self.frame_index)
            else:
                self.frame_index = 0
                #if self.action == 2:
                #self.done_attacking = True

    def update(self):
        if self.action == 2 and not self.done_attacking:
            print("action is 2, attacking: ")
            print(self.action)
            self.action = 2
        elif self.done_attacking:
            self.action = 0
        self.update_animation()

    def draw(self, surface):
        surface.blit(pygame.transform.flip(self.image, False, False), self.rect)


class KingBoss(pygame.sprite.Sprite):
    def __init__(self, x, y, scale):
        super().__init__()

        # animation
        self.action = 0
        self.frame_index = 0
        self.frame_time = pygame.time.get_ticks()
        self.animation_cooldown = 50
        self.animation_cycle = []
        self.animation_types = ['idle', 'death']
        for animation in self.animation_types:
            temp = []
            num_of_frames = len(listfolders(f'images/enemy/Boss/{animation}'))
            for i in range(num_of_frames):
                file_name = "tile0"
                if i < 10:
                    file_name += str(0)
                file_name += str(i)
                img = pygame.image.load(f'images/enemy/Boss/{animation}/{file_name}.png').convert_alpha()
                img = pygame.transform.scale(img, (img.get_width() * scale, img.get_height() * scale))
                temp.append(img)
            self.animation_cycle.append(temp)
        self.image = self.animation_cycle[self.action][self.frame_index]

        # rectangle
        self.original_x = x
        self.original_y = y
        self.rect = self.image.get_rect()
        self.rect.center = (x, y)

        self.done_animation = False

    def reset(self):
        self.rect = pygame.Rect(self.original_x, self.original_y, self.rect.width, self.rect.height)
        self.action = 0
        self.frame_index = 0

    def update_action_dead(self):
        self.action = 1

    def update_animation(self):
        self.image = self.animation_cycle[self.action][self.frame_index]
        if pygame.time.get_ticks() - self.frame_time > self.animation_cooldown:
            if self.frame_index < len(self.animation_cycle[self.action]) - 1:
                self.frame_time = pygame.time.get_ticks()
                self.frame_index += 1
            else:
                print(self.frame_index)
                print(len(self.animation_cycle[self.action]))
                if self.frame_index == len(self.animation_cycle[self.action]) - 1:
                    print("LAST FRAME")
                    print(self.action)
                    print(self.frame_index)
                    self.done_animation = True
                    print(self.done_animation)
                else:
                    self.frame_index = 0


    def update(self):
        if not self.done_animation:
            self.update_animation()

    def draw(self, surface):
        if not self.done_animation:
            surface.blit(pygame.transform.flip(self.image, True, False), self.rect)

