import sys

import pygame
from pygame import K_SPACE

from constants import *

from helper_function import draw_bg

from components.music import *
from states.complete_menu import complete_menu
from states.death_menu import DeathMenu
from states.main_menu import MainMenu
from states.controls_menu import ControlsMenu
from states.credits_menu import CreditsMenu
from states.pause_menu import PauseMenu
from states.death_menu import DeathMenu
from states.intro import Intro
from states.ending import Ending
from components.world import World, Boss
#from surface_loader import SurfaceLoader

class Game:
    def __init__(self):
        pygame.init()

        if INPUT_METHOD == "JOYSTICK":
            self.joystick_player_1 = pygame.joystick.Joystick(0)
            self.joystick_player_2 = pygame.joystick.Joystick(1)
            self.JOYSTICK_RETURN_KEY = 0 #BLACK
            self.JOYSTICK_SHIELD_KEY = 1 #RED
            self.JOYSTICK_JUMP_KEY = 2 #BLUE
            self.JOYSTICK_ROLL_KEY = 3 #YELLOW
            self.JOYSTICK_PAUSE_KEY = 4
            self.JOYSTICK_COIN_KEY = 5

        self.UP_KEY, self.DOWN_KEY, self.START_KEY, self.BACK_KEY, self.COIN_KEY = False, False, False, False, False
        self.displayedSurface = pygame.Surface((SURFACE_WIDTH, SURFACE_HEIGHT))
        self.screen = pygame.display.set_mode((SCREEN_WIDTH, SCREEN_HEIGHT), flags=pygame.FULLSCREEN | pygame.SCALED)
        self.font_name = 'slkscr.ttf'
        self.death_menu = DeathMenu(self)
        self.pause_menu = PauseMenu(self)
        self.main_menu = MainMenu(self)
        self.controls = ControlsMenu(self)
        self.credits = CreditsMenu(self)
        self.intro = Intro(self)
        self.ending = Ending(self)
        #self.surface_loader = SurfaceLoader()
        #self.curr_menu = self.main_menu

        # start screen
        self.menu_screen = True

        # controls screen
        self.controls_screen = False

        # credits screen
        self.credits_screen = False

        # intro
        self.intro_screen = False

        # pause menu
        self.pause_menu_screen = False

        # death screen
        self.death_menu_screen = False

        # disable_player controls
        self.disable_player = False

        # level complete screen
        self.level_complete = False

        # ending screen
        self.ending_screen = False

        # game variable
        self.scroll_total = 0
        self.moving_left = False
        self.moving_right = False
        self.heart_image = pygame.image.load('images/Item/heart/player_heart.png').convert_alpha()
        self.heart_image = pygame.transform.scale(self.heart_image,
                                              (self.heart_image.get_width() * 2.5, self.heart_image.get_height() * 2.5))
        self.boss_heart_image = pygame.image.load('images/Item/heart/boss_heart.png').convert_alpha()
        self.boss_heart_image = pygame.transform.scale(self.boss_heart_image,
                                                  (self.boss_heart_image.get_width() * 2.5,
                                                   self.boss_heart_image.get_height() * 2.5))
        self.shield_image = pygame.image.load('images/Item/shield/shield.png').convert_alpha()
        self.shield_image = pygame.transform.scale(self.shield_image,
                                               (self.shield_image.get_width() * 1.5, self.shield_image.get_height() * 1.5))
        self.player_hearts_surface = pygame.Surface([SURFACE_WIDTH, SURFACE_HEIGHT], pygame.SRCALPHA, 32).convert_alpha()
        self.player_hearts_surface_loaded = False
        self.player_shields_surface = pygame.Surface([SURFACE_WIDTH, SURFACE_HEIGHT], pygame.SRCALPHA, 32).convert_alpha()
        self.player_shields_surface_loaded = False

        self.boss_hearts_surface = pygame.Surface([SURFACE_WIDTH, SURFACE_HEIGHT], pygame.SRCALPHA, 32).convert_alpha()
        self.boss_hearts_surface_loaded = False

        self.level = 0
        self.world = World()
        self.world.load_data(f'map/levels/level{self.level}_data.csv')
        if INPUT_METHOD == "JOYSTICK":
            self.player, self.boss = self.world.process_data(self.level, self.joystick_player_1)
        else:
            self.player, self.boss = self.world.process_data(self.level)

        self.player_old_heart = self.player.heart
        self.player_old_shieldCount = self.player.shieldCount

        self.boss_old_heart = 0
        if self.boss:
            self.boss_old_heart = self.boss.heart
        #self.player.animation_cycle = self.surface_loader.player_images
        self.screen_scroll = 0
        self.world_loaded = True
        self.debouncing_timer = 0


    def update_player_hearts_surface(self):
        self.player_hearts_surface = pygame.Surface([SURFACE_WIDTH, SURFACE_HEIGHT], pygame.SRCALPHA,
                                                    32).convert_alpha()
        for x in range(self.player.heart):
            self.player_hearts_surface.blit(self.heart_image, (15 + (x * 25), 12))
        self.player_hearts_surface_loaded = True

    def update_player_shields_surface(self):
        self.player_shields_surface = pygame.Surface([SURFACE_WIDTH, SURFACE_HEIGHT], pygame.SRCALPHA,
                                                     32).convert_alpha()
        for x in range(self.player.shieldCount):
            self.player_shields_surface.blit(self.shield_image, (13 + (x * 25), 40))
        self.player_shields_surface_loaded = True

    def update_boss_hearts_surface(self):
        self.boss_hearts_surface = pygame.Surface([SURFACE_WIDTH, SURFACE_HEIGHT], pygame.SRCALPHA,
                                                    32).convert_alpha()
        for x in range(self.boss.heart):
            self.boss_hearts_surface.blit(self.boss_heart_image, (SURFACE_WIDTH - (45 + (x * 25)), 12))
        self.boss_hearts_surface_loaded = True

    def run(self, dt: float):
        if self.menu_screen:
            self.check_events()
            if not self.main_menu.surfacesSetup:
                self.main_menu.surface_setup()
            self.main_menu.display_menu()
            self.reset_keys()
        elif self.controls_screen:
            self.check_events()
            if not self.controls.surfacesSetup:
                self.controls.surface_setup()
            self.controls.display_menu()
            self.reset_keys()
        elif self.credits_screen:
            self.check_events()
            #if not self.credits.surfacesSetup:
            #    self.credits.surface_setup()
            self.credits.display_menu()
            self.reset_keys()
        elif self.intro_screen:
            self.check_events()
            if not self.intro.surfaces_loaded:
                self.intro.load_surfaces()
            self.intro.update(dt)
            self.intro.draw()
            self.reset_keys()
        elif self.ending_screen:
            self.debouncing_timer += 1
            self.check_events()
            if not self.ending.surfaces_loaded:
                self.ending.load_surfaces()
            self.ending.update(dt)
            self.ending.draw()
            self.reset_keys()
        elif self.pause_menu_screen:
            self.debouncing_timer += 1
            self.check_events()
            if not self.pause_menu.surfaces_loaded:
                self.pause_menu.load_surfaces()
            self.pause_menu.display_menu()
            self.reset_keys()
        elif self.death_menu_screen:
            self.debouncing_timer += 1
            print(self.debouncing_timer)
            self.check_events()
            if not self.death_menu.surfaces_loaded:
                self.death_menu.load_surfaces()
            self.death_menu.display_menu()
            self.reset_keys()
        elif self.level_complete:
            self.check_events()
            # How to only load the world once...
            if not self.world_loaded:
                self.level += 1
                if self.level == MAX_LEVELS:
                    self.ending_screen = True
                elif self.level < MAX_LEVELS:
                    self.world.reset_level()
                    self.world = World()
                    self.world.load_data(f'map/levels/level{self.level}_data.csv')
                    if INPUT_METHOD == "JOYSTICK":
                        self.player, self.boss = self.world.process_data(self.level, self.joystick_player_1)
                    else:
                        self.player, self.boss = self.world.process_data(self.level)
                    #reset game variables
                    self.moving_left = False
                    self.moving_right = False
                    self.scroll_total = 0
                    self.screen_scroll = 0
                    # reset keys
                    self.reset_keys()
                    self.world_loaded = True
            self.level_complete = False
        else:
            # UPDATE
            #print(self.world)
            #print(self.player)
            self.check_events()
            #print("DT")
            #print(dt)
            #print("PLAYER BEFORE MOVE")
            #print("Get rect coordinaates of player:")
            #print(self.player.rect)
            do_not_scroll = False
            if self.boss:
                do_not_scroll = True
            self.screen_scroll, self.scroll_total = self.player.move(self.moving_left, self.moving_right, self.world,
                                                                     self.world.spike_group, dt, self.scroll_total, do_not_scroll)
            #print("PLAYER AFTER MOVE")
            #print("Get rect coordinaates of player:")
            #print(self.player.rect)
            #print("PLAYER BEFORE UPDATE")
            #print("Alive status")
            #print(self.player.alive)
            #print("Get rect coordinaates of player:")
            #print(self.player.rect)
            self.player.update(dt)
            #print("PLAYER AFTER UPDATE")
            #print("Alive status")
            #print(self.player.alive)
            #print("Get rect coordinaates of player:")
            #print(self.player.rect)
            if self.player.alive:
                if self.player.shield:
                    self.player.update_action(5)
                elif self.player.roll:
                    self.player.update_action(6)
                elif self.player.in_air:
                    self.player.update_action(2)
                elif self.moving_left or self.moving_right:
                    self.player.update_action(1)
                elif self.player.attack:
                    self.player.update_action(3)
                    player_attack_sound.play()
                elif self.player.is_hit:
                    player_hurt_sound.play()
                else:
                    self.player.update_action(0)
            else:
                self.screen_scroll = 0
                #bg_scroll = 0
                self.death_menu_screen = True
            self.world.update(self.level, self.scroll_total, self.screen_scroll)
            for king in self.world.king_group:
                #print("King update")
                king.update(self.player, self.screen_scroll, dt)
                if king.meet_player:
                    for enemy in self.world.enemy_group:
                        enemy.alive = False
                    self.disable_player = True
                    self.moving_left = False
                    self.moving_right = False
                if king.next_level:
                    self.disable_player = False
                    self.world_loaded = False
                    self.level_complete = True
            #print(self.level_complete)
            if self.boss:
                self.boss.update(self.player, self.world, self.screen_scroll, dt)
                if not self.boss.alive:
                    self.world_loaded = False
                    self.level_complete = True
            self.world.enemy_group.update(self.player, self.world, self.screen_scroll, dt)
            self.world.decoration_group.update(self.screen_scroll)
            self.world.item_boxes_group.update(self.player, self.screen_scroll)
            self.world.spike_group.update(self.screen_scroll)

            #Check if player hearts has changed after all the update steps
            if self.player.heart != self.player_old_heart:
                self.player_hearts_surface_loaded = False

            # Check if player shields has changed after all the update steps
            if self.player.shieldCount != self.player_old_shieldCount:
                self.player_shields_surface_loaded = False

            # CHeck if boss hearts has changed after all the update steps and update the old heart of boss
            if self.boss:
                if self.boss.heart != self.boss_old_heart:
                    self.boss_hearts_surface_loaded = False
                self.boss_old_heart = self.boss.heart

            # Update the old heart and shield count to the current count of hearts and shields
            self.player_old_heart = self.player.heart
            self.player_old_shieldCount = self.player.shieldCount

            #if not self.world.obstacle_surface_loaded:
            #    print("LOADING OBSTACLE SURFACE")
            #    self.world.load_obstacle_surface()
            draw_bg(self.world.sky_img, self.displayedSurface)
            #draw_bg(self.world.img2, self.displayedSurface)
            self.world.draw(self.displayedSurface)

            #print("DRAW PLAYER")
            #print("DRAW Get rect coordinaates of player:")
            #print(self.player.rect)
            self.player.draw(self.displayedSurface)
            self.world.enemy_group.draw(self.displayedSurface)
            if self.boss:
                self.boss.draw(self.displayedSurface)
            for king in self.world.king_group:
                king.draw(self.displayedSurface)
                if king.meet_player:
                    king.draw_dialog(self.displayedSurface)
            self.world.decoration_group.draw(self.displayedSurface)
            self.world.item_boxes_group.draw(self.displayedSurface)
            self.world.spike_group.draw(self.displayedSurface)

            if not self.player_hearts_surface_loaded:
                print("LOADING HEARTS SURFACE")
                self.update_player_hearts_surface()
            self.displayedSurface.blit(self.player_hearts_surface, (0, 0))
            if not self.player_shields_surface_loaded:
                print("LOADING SHIELDS SURFACE")
                self.update_player_shields_surface()
            self.displayedSurface.blit(self.player_shields_surface, (0, 0))
            if self.boss:
                if not self.boss_hearts_surface_loaded:
                    print("LOADING BOSS HEARTS SURFACE")
                    self.update_boss_hearts_surface()
                self.displayedSurface.blit(self.boss_hearts_surface, (0, 0))

            for sprite in self.world.all_sprites:
                if sprite.rect.x < self.screen.get_rect().x:
                    sprite.kill()
            #self.screen.blit(pygame.transform.scale(self.displayedSurface, (SCREEN_WIDTH, SCREEN_HEIGHT)), (0, 0))
            self.screen.blit(self.displayedSurface, (0,0))
            


    def check_events(self):
        if INPUT_METHOD == "JOYSTICK":
            for event in pygame.event.get():
                if event.type == pygame.QUIT:
                    pygame.quit()
                    sys.exit()
                if self.menu_screen:
                    if event.type == pygame.JOYBUTTONDOWN:
                        print("MENU SCREEN BUTTON PRESS")
                        if event.button == self.JOYSTICK_RETURN_KEY:
                            self.START_KEY = True
                        if event.button == self.JOYSTICK_COIN_KEY:
                        #if pygame.joystick.Joystick(1).get_button(self.JOYSTICK_COIN_KEY):
                            coin_button_sound.play()
                            self.COIN_KEY = True
                            print("self.COIN_KEY")
                            print(self.COIN_KEY)
                    elif event.type == pygame.JOYAXISMOTION:
                        if event.axis == 1 and event.value > 0.95:
                            cursor_move_sound.play()
                            self.DOWN_KEY = True
                        if event.axis == 1 and event.value < -0.95:
                            cursor_move_sound.play()
                            self.UP_KEY = True
                elif self.controls_screen:
                    if event.type == pygame.JOYBUTTONDOWN:
                        if event.button == self.JOYSTICK_PAUSE_KEY:
                            back_button_sound.play()
                            self.BACK_KEY = True
                    elif event.type == pygame.JOYAXISMOTION:
                        if event.axis == 1 and event.value > 0.95:
                            cursor_move_sound.play()
                            self.DOWN_KEY = True
                        if event.axis == 1 and event.value < -0.95:
                            cursor_move_sound.play()
                            self.UP_KEY = True
                elif self.credits_screen:
                    if event.type == pygame.JOYBUTTONDOWN:
                        if event.button == self.JOYSTICK_PAUSE_KEY:
                            back_button_sound.play()
                            self.BACK_KEY = True
                elif self.intro_screen:
                    if event.type == pygame.JOYBUTTONDOWN:
                        if event.button == self.JOYSTICK_RETURN_KEY:
                            self.START_KEY = True
                elif self.ending_screen:
                    if self.debouncing_timer > 30:
                        if event.type == pygame.JOYBUTTONDOWN:
                            if event.button == self.JOYSTICK_RETURN_KEY:
                                self.debouncing_timer = 0
                                select_sound.play()
                                self.START_KEY = True
                    if self.debouncing_timer > 15:
                        if event.type == pygame.JOYAXISMOTION:
                            if event.axis == 1 and event.value > 0.95:
                                self.debouncing_timer = 0
                                cursor_move_sound.play()
                                self.DOWN_KEY = True
                            if event.axis == 1 and event.value < -0.95:
                                self.debouncing_timer = 0
                                cursor_move_sound.play()
                                self.UP_KEY = True
                elif self.pause_menu_screen:
                    if self.debouncing_timer > 30:
                        if event.type == pygame.JOYBUTTONDOWN:
                            if event.button == self.JOYSTICK_RETURN_KEY:
                                self.debouncing_timer = 0
                                select_sound.play()
                                self.START_KEY = True
                    if self.debouncing_timer > 15:
                        if event.type == pygame.JOYAXISMOTION:
                            if event.axis == 1 and event.value > 0.95:
                                self.debouncing_timer = 0
                                cursor_move_sound.play()
                                self.DOWN_KEY = True
                            if event.axis == 1 and event.value < -0.95:
                                self.debouncing_timer = 0
                                cursor_move_sound.play()
                                self.UP_KEY = True
                elif self.death_menu_screen:
                    if self.debouncing_timer > 30:
                        if event.type == pygame.JOYBUTTONDOWN:
                            if event.button == self.JOYSTICK_RETURN_KEY:
                                print("ENTER KEY PRESSED ON DEATH SCREEN")
                                select_sound.play()
                                self.debouncing_timer = 0
                                self.START_KEY = True
                    if self.debouncing_timer > 15:
                        if event.type == pygame.JOYAXISMOTION:
                            if event.axis == 1 and event.value > 0.95:
                                self.debouncing_timer = 0
                                cursor_move_sound.play()
                                self.DOWN_KEY = True
                            if event.axis == 1 and event.value < -0.95:
                                self.debouncing_timer = 0
                                cursor_move_sound.play()
                                self.UP_KEY = True
                elif self.disable_player:
                    pass
                elif self.level_complete:
                    pass

                        # if main_button.checkForInput(Menu_Mouse_POS):
                        #    mouse_click_sound.play()
                        # self.menu_screen = True
                        # self.moving_left = False
                        # self.moving_right = False
                        # self.level_complete = False
                        # reset_level()
                        # self.world = World()
                        # self.player = self.world.process_data(self.screen_scroll)
                        # if next_level_button.checkForInput(Menu_Mouse_POS):
                        #    mouse_click_sound.play()
                        #    pass
                else:
                    if event.type == pygame.JOYBUTTONDOWN:
                        if event.button == self.JOYSTICK_PAUSE_KEY:
                            pause_sound.play()
                            self.pause_menu_screen = True
                        if not self.player.shield and not self.player.roll:
                            if self.joystick_player_1.get_button(self.JOYSTICK_ROLL_KEY) and not self.player.in_air:
                                self.player.roll = True
                                player_roll_sound.play()
                            elif self.joystick_player_1.get_button(self.JOYSTICK_JUMP_KEY) and self.player.alive:
                                self.player.jump = True
                                player_jump_sound.play()
                            elif self.joystick_player_1.get_button(self.JOYSTICK_RETURN_KEY) and not self.joystick_player_1.get_button(self.JOYSTICK_SHIELD_KEY) and not self.player.in_air:
                                self.player.attack = True
                                player_attack_sound.play()
                            elif not self.joystick_player_1.get_button(self.JOYSTICK_RETURN_KEY) and self.joystick_player_1.get_button(self.JOYSTICK_SHIELD_KEY):
                                print("shield button down")
                                self.player.shielding(True)
                                player_shield_sound.play(-1)
                    if event.type == pygame.JOYBUTTONUP:
                        if self.joystick_player_1.get_button(self.JOYSTICK_SHIELD_KEY):
                            print("shield button up")
                            self.player.shielding(False)
                            player_shield_sound.stop()
                    if event.type == pygame.JOYAXISMOTION:
                        if not self.player.shield and not self.player.roll:
                            if self.joystick_player_1.get_axis(0) and event.value < -0.95:
                                self.moving_left = True
                                self.moving_right = False
                                self.player.attack = False
                                print("left joystick")
                                #running_sound.play()
                            elif self.joystick_player_1.get_axis(0) and event.value > 0.95:
                                self.moving_right = True
                                self.moving_left = False
                                self.player.attack = False
                                print("right joystick")
                                #running_sound.play()
                            else:
                                self.moving_left = False
                                self.moving_right = False
                                print("not moving left or right")
                    

        elif INPUT_METHOD == "KEYBOARD":
            for event in pygame.event.get():
                if event.type == pygame.KEYDOWN:
                    if event.key == pygame.K_ESCAPE:
                        pygame.quit()
                        sys.exit()
                if event.type == pygame.QUIT:
                    pygame.quit()
                    sys.exit()
                if self.menu_screen:
                    if event.type == pygame.KEYDOWN:
                        if event.key == pygame.K_SPACE:
                            print("Enter key pressed on menu screen")
                            self.START_KEY = True
                        if event.key == pygame.K_BACKSPACE:
                            back_button_sound.play()
                            self.BACK_KEY = True
                        if event.key == pygame.K_DOWN:
                            cursor_move_sound.play()
                            self.DOWN_KEY = True
                        if event.key == pygame.K_UP:
                            cursor_move_sound.play()
                            self.UP_KEY = True
                        if event.key == pygame.K_c:
                            coin_button_sound.play()
                            print("Coin key on menu screen")
                            self.COIN_KEY = True
                elif self.controls_screen:
                    if event.type == pygame.KEYDOWN:
                        if event.key == pygame.K_BACKSPACE:
                            back_button_sound.play()
                            self.BACK_KEY = True
                        if event.key == pygame.K_DOWN:
                            cursor_move_sound.play()
                            self.DOWN_KEY = True
                        if event.key == pygame.K_UP:
                            cursor_move_sound.play()
                            self.UP_KEY = True
                elif self.credits_screen:
                    if event.type == pygame.KEYDOWN:
                        if event.key == pygame.K_BACKSPACE:
                            back_button_sound.play()
                            self.BACK_KEY = True
                elif self.intro_screen:
                    if event.type == pygame.KEYDOWN:
                        if event.key == pygame.K_SPACE:
                            self.START_KEY = True
                elif self.pause_menu_screen:
                    if event.type == pygame.KEYDOWN:
                        if event.key == pygame.K_SPACE:
                            select_sound.play()
                            self.START_KEY = True
                        if event.key == pygame.K_DOWN:
                            cursor_move_sound.play()
                            self.DOWN_KEY = True
                        if event.key == pygame.K_UP:
                            cursor_move_sound.play()
                            self.UP_KEY = True
                elif self.death_menu_screen or self.ending_screen:
                    if event.type == pygame.KEYDOWN:
                        if event.key == pygame.K_SPACE:
                            select_sound.play()
                            self.START_KEY = True
                        if event.key == pygame.K_DOWN:
                            cursor_move_sound.play()
                            self.DOWN_KEY = True
                        if event.key == pygame.K_UP:
                            cursor_move_sound.play()
                            self.UP_KEY = True
                elif self.disable_player:
                    pass
                elif self.level_complete:
                    pass

                    # if main_button.checkForInput(Menu_Mouse_POS):
                    #    mouse_click_sound.play()
                    # self.menu_screen = True
                    # self.moving_left = False
                    # self.moving_right = False
                    # self.level_complete = False
                    # reset_level()
                    # self.world = World()
                    # self.player = self.world.process_data(self.screen_scroll)
                    # if next_level_button.checkForInput(Menu_Mouse_POS):
                    #    mouse_click_sound.play()
                    #    pass
                else:
                    if event.type == pygame.KEYDOWN:
                        #if event.key == pygame.K_s:
                        #    pygame.image.save(self.screen, "heroshot.jpg")
                        if event.key == pygame.K_m:
                            pause_sound.play()
                            self.pause_menu_screen = True
                        if not self.player.shield and not self.player.roll:
                            if event.key == pygame.K_LEFT:
                                self.moving_left = True
                                self.moving_right = False
                                self.player.attack = False
                                #running_sound.play()
                            if event.key == pygame.K_RIGHT:
                                self.moving_right = True
                                self.moving_left = False
                                self.player.attack = False
                                #running_sound.play()
                            if event.key == pygame.K_a and not self.player.in_air:
                                self.player.roll = True
                                player_roll_sound.play()
                            if event.key == pygame.K_UP and self.player.alive:
                                self.player.jump = True
                                player_jump_sound.play()
                            if event.key == pygame.K_ESCAPE:
                                print("escape")
                                # Run = false
                            if event.key == pygame.K_SPACE and event.key != pygame.K_LSHIFT and not self.player.in_air:
                                self.player.attack = True
                                player_attack_sound.play()
                            if event.key != pygame.K_SPACE and event.key == pygame.K_LSHIFT:
                                self.player.shielding(True)
                                player_shield_sound.play(-1)

                    if event.type == pygame.KEYUP:
                        if event.key == pygame.K_LEFT:
                            self.moving_left = False
                            #running_sound.stop()
                        if event.key == pygame.K_RIGHT:
                            self.moving_right = False
                            #running_sound.stop()
                        if event.key == pygame.K_ESCAPE:
                            print("escape")
                            # run = False
                        if event.key == pygame.K_LSHIFT:
                            self.player.shielding(False)
                            player_shield_sound.stop()

    def reset_keys(self):
        self.UP_KEY, self.DOWN_KEY, self.START_KEY, self.BACK_KEY, self.COIN_KEY = False, False, False, False, False




