import pygame

from constants import *
from helper_function import draw_bg, create_text_surf, listfolders

class Knight(pygame.sprite.Sprite):
    def __init__(self, x, y, scale):
        super().__init__()

        # animation
        self.action = 1
        self.frame_index = 0
        self.frame_time = pygame.time.get_ticks()
        self.animation_cooldown = 50
        self.animation_cycle = []
        self.animation_types = ['idle', 'walk']
        for animation in self.animation_types:
            temp = []
            num_of_frames = len(listfolders(f'images/player/knight/{animation}'))
            for i in range(num_of_frames):
                img = pygame.image.load(f'images/player/knight/{animation}/tile{i}.png').convert_alpha()
                img = pygame.transform.scale(img, (img.get_width() * scale, img.get_height() * scale))
                temp.append(img)
            self.animation_cycle.append(temp)
        self.image = self.animation_cycle[self.action][self.frame_index]

        # rectangle
        self.original_x = x
        self.original_y = y
        self.rect = self.image.get_rect()
        self.rect.center = (x, y)

    def reset(self):
        self.rect = pygame.Rect(self.original_x, self.original_y, self.rect.width, self.rect.height)
        self.action = 1
        self.frame_index = 0

    def move(self, end):
        if self.action == 1:
            self.rect.x = self.rect.x + 4
        if self.rect.x >= end:
            self.action = 0

    def update_animation(self):
        self.image = self.animation_cycle[self.action][self.frame_index]
        if pygame.time.get_ticks() - self.frame_time > self.animation_cooldown:
            if self.frame_index < len(self.animation_cycle[self.action]) - 1:
                self.frame_time = pygame.time.get_ticks()
                self.frame_index += 1
            else:
                self.frame_index = 0

    def update(self):
        self.update_animation()

    def draw(self, surface):
        surface.blit(pygame.transform.flip(self.image, False, False), self.rect)

