import pygame

from ending_helper import Knight, KingBoss
from constants import *
from helper_function import *
from states.menu import Menu
from components.world import World


class Ending(Menu):
    def __init__(self, game):
        Menu.__init__(self, game)
        self.state = "Return to Main Menu"
        self.game = game
        self.knight_ending = Knight(0.2*SURFACE_WIDTH, SURFACE_HEIGHT - 60, 2)
        self.kingboss_ending = KingBoss(0.6*SURFACE_WIDTH, SURFACE_HEIGHT - 60, 2)
        self.dialog_count = 0
        self.surfaces_loaded = False
        self.backgroundSurface = pygame.Surface((SURFACE_WIDTH, SURFACE_HEIGHT))
        self.draw_menu = False

        self.quitx, self.quity = 0.5 * self.mid_w, 1 * self.mid_h
        self.cursor_rect = pygame.Rect(0, 0, 30, 30)
        self.cursor_surf = create_text_surf('*', self.game.font_name, 30, WHITE)
        self.cursor_rect.midtop = (self.quitx + self.offset, self.quity)
        self.menu_surface = pygame.Surface([SURFACE_WIDTH, SURFACE_HEIGHT], pygame.SRCALPHA,
                       32).convert_alpha()
        # tmp = pygame.Surface((SURFACE_WIDTH, SURFACE_HEIGHT), pygame.SRCALPHA)
        # tmp.fill((255, 255, 255, 0))
        # self.menu_surface.blit(tmp, (0, 0), special_flags=pygame.BLEND_RGBA_MULT)
        self.pause_menu_text_surf = create_text_surf('YOU WIN!', 'slkscrb.ttf', 30, WHITE)
        self.quit_text_surf = create_text_surf("Return to Main Menu", self.game.font_name, 30, WHITE)
        self.menu_surface.blit(self.pause_menu_text_surf, (SURFACE_WIDTH * 0.35, SURFACE_HEIGHT / 8))
        self.menu_surface.blit(self.quit_text_surf, (self.quitx, self.quity))
        #self.oldDisplayedSurface = pygame.Surface((SURFACE_WIDTH, SURFACE_HEIGHT)).convert_alpha()

    def reset(self):
        self.knight_ending.reset()
        self.dialog_count = 0

    def update(self, dt):
        # Check for
        self.check_input()
        self.knight_ending.update()
        self.kingboss_ending.update()
        self.knight_ending.move(self.kingboss_ending.rect.left - self.knight_ending.rect.width - 20)
        #if self.knight_ending.done_attacking:
        #    self.kingboss_ending.update_action_dead()
        if self.knight_ending.action == 0:
            self.kingboss_ending.update_action_dead()
            self.draw_menu = True


    def check_input(self):
        if self.game.START_KEY and self.knight_ending.action == 0:
            self.surfaces_loaded = False
            self.game.ending_screen = False
            # Reset world, dont know if this is the right place
            self.game.world.reset_level()
            self.game.level = 0
            self.game.world = World()
            self.game.world.load_data(f'map/levels/level{self.game.level}_data.csv')
            if INPUT_METHOD == "JOYSTICK":
                self.game.player, self.game.boss = self.game.world.process_data(self.game.level,
                                                                                self.game.joystick_player_1)
            else:
                self.game.player, self.game.boss = self.game.world.process_data(self.game.level)
            # reset game variables
            self.game.moving_right = False
            self.game.moving_left = False
            self.game.scroll_total = 0
            self.game.screen_scroll = 0
            # reset keys
            self.game.reset_keys()

            self.game.menu_screen = True
            #print("Enter key pressed")
            #self.knight_ending.frame_index = 1
            #self.knight_ending.action = 2
            #if self.dialog_count == 7:
            #    self.game.intro_screen = False



    def load_surfaces(self):
        bg_img = pygame.image.load('images/Background/forest_background.png').convert_alpha()
        bg_img = pygame.transform.scale(bg_img, (SURFACE_WIDTH, SURFACE_HEIGHT))
        draw_bg(bg_img, self.backgroundSurface)
        self.surfaces_loaded = True

    def draw(self):
        self.game.displayedSurface.blit(self.backgroundSurface, (0, 0))
        self.knight_ending.draw(self.game.displayedSurface)
        self.kingboss_ending.draw(self.game.displayedSurface)
        #if self.dialog_count > 0:
        #    self.drawDialog(self.game.displayedSurface, self.dialog_count)
        #self.game.screen.blit(pygame.transform.scale(self.game.displayedSurface, self.game.screen.get_rect().size), (0, 0))
        if self.draw_menu:
            self.game.displayedSurface.blit(self.menu_surface, (0, 0))
            self.game.displayedSurface.blit(self.cursor_surf, self.cursor_rect)
        self.game.screen.blit(self.game.displayedSurface, (0, 0))
