import pygame

from intro_helper import Knight
from components.king import King
from constants import *
from helper_function import *
from components.music import select_sound


class Intro():
    def __init__(self, game):
        self.game = game
        self.knight_intro = Knight(0.1*SURFACE_WIDTH, SURFACE_HEIGHT - 85, 2)
        self.king_intro = King(0.8*SURFACE_WIDTH, SURFACE_HEIGHT - 50, 2, 0)
        self.dialog_count = 0
        self.surfaces_loaded = False
        self.backgroundSurface = pygame.Surface((SURFACE_WIDTH, SURFACE_HEIGHT))
        self.dialog1_surf = create_text_surf("Your majesty.", 'slkscr.ttf', 10, WHITE)
        self.dialog2_surf = create_text_surf("My dear knight, our world is in danger!", 'slkscr.ttf', 10, WHITE)
        self.dialog3_surf = create_text_surf("An unknown adversary has invaded with a legion of monsters.", 'slkscr.ttf', 10, WHITE)
        self.dialog4_surf = create_text_surf("And only you can save us from this threat!", 'slkscr.ttf', 10, WHITE)
        self.dialog5_surf = create_text_surf("For the sake of us all, find and defeat this evil man.", 'slkscr.ttf', 10, WHITE)
        self.dialog6_surf = create_text_surf("I will protect you and the people with my life, your majesty.", 'slkscr.ttf', 10, WHITE)

    def reset(self):
        self.knight_intro.reset()
        self.dialog_count = 0

    def update(self, dt):
        # Check for
        self.check_input()
        self.knight_intro.update()
        self.king_intro.update(self.game.player, 0, dt)
        self.knight_intro.move(SURFACE_WIDTH * 0.4)

    def check_input(self):
        if self.game.START_KEY and self.knight_intro.rect.x >= (SURFACE_WIDTH * 0.4):
            select_sound.play()
            self.dialog_count += 1
            if self.dialog_count == 7:
                self.game.intro_screen = False

    def load_surfaces(self):
        print("LOADING SURFACES")
        bg_img = pygame.image.load('images/Background/forest_background.png').convert_alpha()
        bg_img = pygame.transform.scale(bg_img, (SURFACE_WIDTH, SURFACE_HEIGHT))
        draw_bg(bg_img, self.backgroundSurface)
        self.surfaces_loaded = True

    def drawDialog(self, surface, dialog_count):
        if dialog_count == 1:
            surface.blit(self.dialog1_surf, (self.knight_intro.rect.x, 0.9 * self.knight_intro.rect.y))
        elif dialog_count == 2:
            surface.blit(self.dialog2_surf, (0.8 * self.king_intro.rect.x, 0.9 * self.king_intro.rect.y))
        elif dialog_count == 3:
            surface.blit(self.dialog3_surf, (0.5 * self.king_intro.rect.x, 0.9 * self.king_intro.rect.y))
        elif dialog_count == 4:
            surface.blit(self.dialog4_surf, (0.7 * self.king_intro.rect.x, 0.9 * self.king_intro.rect.y))
        elif dialog_count == 5:
            surface.blit(self.dialog5_surf, (0.6 * self.king_intro.rect.x, 0.9 * self.king_intro.rect.y))
        elif dialog_count == 6:
            surface.blit(self.dialog6_surf, (0.4 * self.knight_intro.rect.x, 0.9 * self.knight_intro.rect.y))

    def draw(self):
        self.game.displayedSurface.blit(self.backgroundSurface, (0, 0))
        self.knight_intro.draw(self.game.displayedSurface)
        self.king_intro.draw(self.game.displayedSurface)
        if self.dialog_count > 0:
            self.drawDialog(self.game.displayedSurface, self.dialog_count)
        #self.game.screen.blit(pygame.transform.scale(self.game.displayedSurface, self.game.screen.get_rect().size), (0, 0))
        self.game.screen.blit(self.game.displayedSurface, (0, 0))
