import pygame
from states.menu import Menu
from helper_function import draw_bg, create_text_surf
from constants import *
from components.music import select_sound, denied_sound
#from setup import *


class MainMenu(Menu):
    def __init__(self, game):
        Menu.__init__(self, game)
        self.coin_inserted = False
        self.state = "Play"
        self.surfacesSetup = False
        self.playx, self.playy = 0.8 * self.mid_w, 0.75*self.mid_h
        #self.controlsx, self.controlsy = 0.8 * self.mid_w, self.mid_h
        self.creditsx, self.creditsy = 0.8 * self.mid_w, 1.25*self.mid_h
        self.cursor_rect.midtop = (self.playx + self.offset, self.playy)
        self.backgroundSurface = pygame.Surface((SURFACE_WIDTH, SURFACE_HEIGHT))
        self.insert_coin_text_surf = create_text_surf("Insert Coin to Play", self.game.font_name, 30, YELLOW)
        self.ready_play_text_surf = create_text_surf("READY TO PLAY", self.game.font_name, 30, YELLOW)



    def surface_setup(self):
        forest_img = pygame.image.load('images/Background/Grassy_Mountains_preview_fullcolor.png').convert_alpha()
        forest_img = pygame.transform.scale(forest_img, (SURFACE_WIDTH, SURFACE_HEIGHT))
        draw_bg(forest_img, self.backgroundSurface)
        main_menu_text_surf = create_text_surf('Main Menu', 'slkscrb.ttf', 30, WHITE)
        play_text_surf = create_text_surf("Play", self.game.font_name, 30, WHITE)
        #controls_text_surf = create_text_surf("Controls", self.game.font_name, 30, WHITE)
        credits_text_surf = create_text_surf("Credits", self.game.font_name, 30, WHITE)
        self.backgroundSurface.blit(main_menu_text_surf, (SURFACE_WIDTH / 3, SURFACE_HEIGHT / 8))
        self.backgroundSurface.blit(play_text_surf, (self.playx, self.playy))
        #self.backgroundSurface.blit(controls_text_surf, (self.controlsx, self.controlsy))
        self.backgroundSurface.blit(credits_text_surf, (self.creditsx, self.creditsy))
        self.surfacesSetup = True

    def display_menu(self):
        self.check_input()
        self.game.displayedSurface.blit(self.backgroundSurface, (0,0))
        if not self.coin_inserted:
            self.game.displayedSurface.blit(self.insert_coin_text_surf, (SURFACE_WIDTH * 0.25, SURFACE_HEIGHT * 7/8))
        else:
            self.game.displayedSurface.blit(self.ready_play_text_surf, (SURFACE_WIDTH * 0.31, SURFACE_HEIGHT * 7/8))
        self.game.displayedSurface.blit(self.cursor_surf, self.cursor_rect)
        self.blit_screen()

    def move_cursor(self):
        if self.game.DOWN_KEY:
            if self.state == 'Play':
                self.cursor_rect.midtop = (self.creditsx + self.offset, self.creditsy)
                self.state = 'Credits'
            # elif self.state == 'Controls':
            #     self.cursor_rect.midtop = (self.creditsx + self.offset, self.creditsy)
            #     self.state = 'Credits'
            elif self.state == 'Credits':
                self.cursor_rect.midtop = (self.playx + self.offset, self.playy)
                self.state = 'Play'
        elif self.game.UP_KEY:
            if self.state == 'Play':
                self.cursor_rect.midtop = (self.creditsx + self.offset, self.creditsy)
                self.state = 'Credits'
            # elif self.state == 'Controls':
            #     self.cursor_rect.midtop = (self.playx + self.offset, self.playy)
            #     self.state = 'Play'
            elif self.state == 'Credits':
                self.cursor_rect.midtop = (self.playx + self.offset, self.playy)
                self.state = 'Play'

    def check_input(self):
        self.move_cursor()
        if self.game.COIN_KEY:
            self.coin_inserted = True
            print("coin inserted")
        if self.game.START_KEY:
            print("start key was pressed. check coini inserted")
            print(self.coin_inserted)
            if self.state == 'Play' and not self.coin_inserted:
                denied_sound.play()
            if self.state == 'Play' and self.coin_inserted:
                select_sound.play()
                self.game.menu_screen = False
                self.coin_inserted = False
                self.game.intro.reset()
                self.game.intro_screen = True
            # elif self.state == 'Controls':
            #     self.game.menu_screen = False
            #     self.game.controls_screen = True
            elif self.state == 'Credits':
                select_sound.play()
                self.game.menu_screen = False
                self.game.credits_screen = True
            #self.run_display = False
